from django.db import models
from paranoid_model import models as paranoid_model


class Person(paranoid_model.Paranoid):
    """
    Person model with Paranoid inheritance
    Attributes:
         name: CharField
         created_at: DateTimeField
         updated_at: DateTimeField
         deleted_at: DateTimeField
    """
    name = models.CharField(max_length=255)


class Phone(paranoid_model.Paranoid):
    """
    Phone model with Paranoid inheritance
    Attributes:
         phone: CharField
         owner: ForeignKey to Person
         created_at: DateTimeField
         updated_at: DateTimeField
         deleted_at: DateTimeField
    """

    phone = models.CharField(max_length=255)
    owner = models.ForeignKey(Person, on_delete=models.CASCADE, related_name='phones')


class Address(paranoid_model.Paranoid):
    """
    Address model with Paranoid inheritance
    Attributes:
         street: CharField
         owner: ForeignKey to Person
         created_at: DateTimeField
         updated_at: DateTimeField
         deleted_at: DateTimeField
    """
    street = models.CharField(max_length=255)
    owner = models.ForeignKey(Person, on_delete=models.CASCADE, related_name='addresses')


class Car(paranoid_model.Paranoid):
    owner = models.OneToOneField(Person, on_delete=models.CASCADE)


class Clothes(models.Model):
    """
    Person clothes not paranoid model
    Attributes:
        street: CharField
        owner: ForeignKey to Person
        created_at: DateTimeField
        updated_at: DateTimeField
        deleted_at: DateTimeField
    """
    description = models.CharField(max_length=255)
    person = models.ForeignKey(Person, on_delete=models.CASCADE, related_name='my_clothes')



from django.test import TestCase
from paranoid_model.tests.models import Car, Person
from paranoid_model.tests.utils import get_person_instance


class RelatedModelTest(TestCase):
    """Test model with relatioships ManyToMany, ForeignKey, OneToOne"""

    def setUp(self):
        person = get_person_instance()
        person.save()
        car = Car(owner=person)
        car.save()

    def test_create(self):
        """Test create a model with OneToOneField"""
        Car.objects.get()
        person = Person.objects.get()

        self.assertTrue(Car.objects.filter(owner=person).exists())

    def test_delete(self):
        """Test dele model with OneToOneField"""
        person = Person.objects.get()
        car = Car.objects.get()

        person.delete()
        self.assertFalse(Car.objects.all().exists())
        self.assertTrue(Car.objects.filter(with_deleted=True).exists())

        self.assertRaises(Car.SoftDeleted, Car.objects.get)

        car.delete(hard_delete=True)
        self.assertFalse(Car.objects.all(with_deleted=True).exists())

    def test_related_object_been_queried_by_default(self):
        """Test if related object is been queried"""
        person = Person.objects.get()
        person.delete()
        self.assertEquals(Car.objects.all().exists(), False)

        self.assertEquals(Car.objects.get_deleted().owner, person)



from django.contrib import admin
from django.utils.translation import gettext_lazy as _
from django.http import HttpResponseRedirect
from paranoid_model.exceptions import SoftDeleted
from django.core.exceptions import ValidationError
from django.contrib import messages
from django.contrib.admin.actions import delete_selected


class ParanoidAdminFilter(admin.SimpleListFilter):
    """Class to handle filter on site"""
    title = _('soft deleted')
    parameter_name = 'deleted_at'

    def lookups(self, request, model_admin):
        """
        Method with (value, option) for filter form
        on admin view
        """

        return (
            ('not soft', _('Not soft deleted')),
            ('soft', _('Soft deleted')),
        )

    def queryset(self, request, queryset):
        """
        Method to get query set with instances to
        show on site
        """

        with_deleted = False  # exclude deleted if not querrying all

        value = self.value()

        if value is None or value == 'all':
            with_deleted = True
        elif value == 'soft':
            return queryset.deleted_only()

        return queryset.all(with_deleted=with_deleted)


class ParanoidAdmin(admin.ModelAdmin):
    """
    Paranoid base Admin
    """

    change_form_template = 'admin/paranoid_model/change_form.html'

    readonly_fields = ('deleted_at',)
    list_display = ('pk', '__str__', 'created_at', 'updated_at', 'is_not_deleted')
    list_display_links = ('pk', '__str__',)
    list_filter = (ParanoidAdminFilter,)
    actions = ['restore_selected', 'permanently_delete']

    def is_not_deleted(self, obj):
        """
        is not deleted table column on admin list view
        """
        return not obj.is_soft_deleted
    is_not_deleted.boolean = True

    def response_change(self, request, obj):
        """
        Intercept default submit method on change_form view to perform
        custom options
        Args:
            request
            obj: instance
        """

        if '_restore' in request.POST:
            obj.restore()

            level, message = messages.SUCCESS, f'{obj.__str__()} restored.'
            log_message = 'Restored'
            messages.add_message(request, level, message)
        else:
            return super().response_change(request, obj)

        super().log_addition(request, obj, log_message)
        return HttpResponseRedirect('.')

    def delete_view(self, request, object_id, extra_context=None):
        """
        Override default delete_view method to add 'Soft deleted'
        to log history
        """

        _return = super().delete_view(request, object_id, extra_context=None)
        if request.POST:
            try:
                obj = self.model.objects.get(pk=object_id)
            except SoftDeleted:
                obj = self.model.objects.get_deleted(pk=object_id)

            hard_delete = 'hard_delete' in request.GET.keys()
            obj.delete(hard_delete=hard_delete)

            report = 'Permanently deleted.' if hard_delete else 'Soft deleted.'

            super().log_addition(request, obj, report)

        return _return

    def restore_selected(self, request, queryset):
        """
        Action method to restore every instance selected
        """
        count = queryset.restore()
        messages.add_message(request, messages.INFO, f'{count} restored.')

    def permanently_delete(self, request, queryset):
        """
        Action method to hard delete every instance selected
        """
        self.hard_delete = True
        return delete_selected(self, request, queryset)

    def delete_queryset(self, request, queryset):
        """
        Delete a queryset
        """
        queryset.delete(hard_delete=self.hard_delete)
        self.hard_delete = False
    hard_delete = False  # boolean for 'permanently delete' action
    # use django delete confirmation is pretty hard, so instaead of create
    # our own, it is easier to have a boolean variable to check if permanently or not.
    # Django's 'delete_selected' uses delete_queryset() to delete.

    def get_object(self, request, object_id, from_field=None):
        """
        Return instance whit matching parameter, overwitten to work
        without errors with ParanoidQuerySet if instance has been soft deleted
        """

        queryset = super().get_queryset(request)
        model = queryset.model
        field = model._meta.pk if from_field is None else model._meta.get_field(from_field)
        try:
            object_id = field.to_python(object_id)
            return queryset.get(**{field.name: object_id})
        except (model.DoesNotExist, ValidationError, ValueError):
            return None
        except SoftDeleted:
            return queryset.get_deleted(**{field.name: object_id})



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='django-paranoid-model',
    version='1.0.6',
    url='https://github.com/DarknessRdg/django-paranoid-model',
    keywords='django paranoid safedelete softdelete',

    author='Luan Rodrigues',
    author_email='luanrodriguesbusiness@hotmail.com',

    description='Django abstract model with paranoid behavior.',
    long_description=long_description,
    long_description_content_type='text/markdown',

    packages=setuptools.find_packages(),

    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    install_requires=['Django'],
    extra_requires={
        'test': ['Faker']
    },
    python_requires='>=3.4',

    project_urls={
        'Bug Reports': 'https://github.com/DarknessRdg/django-paranoid-model/issues',
        'Source': 'https://github.com/DarknessRdg/django-paranoid-model/'
    },
    include_package_data=True
)
